@@ -18,15 +18,16 @@ import shutil
 from abc import ABC, abstractmethod
 from argparse import ArgumentParser, _ArgumentGroup
 from contextlib import contextmanager
+from threading import Condition
+import time
 from typing import (Any,
-                    Callable,
                     ContextManager,
                     Dict,
-                    Iterator,
                     List,
+                    Set,
+                    Iterator,
                     NamedTuple,
                     Optional,
-                    TypeVar,
                     Union,
                     cast)
 
@@ -44,12 +45,18 @@ logger = logging.getLogger(__name__)
 EXIT_STATUS_UNAVAILABLE_VALUE = 255
 
 class BatchJobExitReason(enum.IntEnum):
-    FINISHED: int = 1  # Successfully finished.
-    FAILED: int = 2  # Job finished, but failed.
-    LOST: int = 3  # Preemptible failure (job's executing host went away).
-    KILLED: int = 4  # Job killed before finishing.
-    ERROR: int = 5  # Internal error.
-    MEMLIMIT: int = 6  # Job hit batch system imposed memory limit
+    FINISHED: int = 1
+    """Successfully finished."""
+    FAILED: int = 2
+    """Job finished, but failed."""
+    LOST: int = 3
+    """Preemptable failure (job's executing host went away)."""
+    KILLED: int = 4
+    """Job killed before finishing."""
+    ERROR: int = 5
+    """Internal error."""
+    MEMLIMIT: int = 6
+    """Job hit batch system imposed memory limit."""
 
 class UpdatedBatchJobInfo(NamedTuple):
     jobID: int
@@ -88,7 +95,7 @@ class AbstractBatchSystem(ABC):
         """
         Whether this batch system supports auto-deployment of the user script itself.
 
-        If it does, the :meth:`.setUserScript` can be invoked to set the resource
+        If it does, the :meth:`setUserScript` can be invoked to set the resource
         object representing the user script.
 
         Note to implementors: If your implementation returns True here, it should also override
@@ -227,13 +234,15 @@ class AbstractBatchSystem(ABC):
 
     def setEnv(self, name: str, value: Optional[str] = None) -> None:
         """
-        Set an environment variable for the worker process before it is launched. The worker
-        process will typically inherit the environment of the machine it is running on but this
-        method makes it possible to override specific variables in that inherited environment
-        before the worker is launched. Note that this mechanism is different to the one used by
-        the worker internally to set up the environment of a job. A call to this method affects
-        all jobs issued after this method returns. Note to implementors: This means that you
-        would typically need to copy the variables before enqueuing a job.
+        Set an environment variable for the worker process before it is launched.
+
+        The worker process will typically inherit the environment of the machine
+        it is running on but this method makes it possible to override specific
+        variables in that inherited environment before the worker is launched.
+        Note that this mechanism is different to the one used by the worker
+        internally to set up the environment of a job. A call to this method affects
+        all jobs issued after this method returns. Note to implementors: This
+        means that you would typically need to copy the variables before enqueuing a job.
 
         If no value is provided it will be looked up from the current environment.
         """
@@ -360,10 +369,10 @@ class BatchSystemSupport(AbstractBatchSystem):
 
         If no value is provided it will be looked up from the current environment.
 
-        :param str name: the environment variable to be set on the worker.
+        :param name: the environment variable to be set on the worker.
 
-        :param str value: if given, the environment variable given by name will be set to this value.
-               if None, the variable's current value will be used as the value on the worker
+        :param value: if given, the environment variable given by name will be set to this value.
+        if None, the variable's current value will be used as the value on the worker
 
         :raise RuntimeError: if value is None and the name cannot be found in the environment
         """
@@ -383,14 +392,30 @@ class BatchSystemSupport(AbstractBatchSystem):
         # We do in fact send messages to the message bus.
         self._outbox = message_bus.outbox()
 
-    def formatStdOutErrPath(self, toil_job_id: int, cluster_job_id: str, std: str) -> str:
+    def get_batch_logs_dir(self) -> str:
+        """
+        Get the directory where the backing batch system should save its logs.
+
+        Only really makes sense if the backing batch system actually saves logs
+        to a filesystem; Kubernetes for example does not. Ought to be a
+        directory shared between the leader and the workers, if the backing
+        batch system writes logs onto the worker's view of the filesystem, like
+        many HPC schedulers do.
+        """
+        if self.config.batch_logs_dir:
+            # Use what is specified
+            return self.config.batch_logs_dir
+        # And if nothing is specified use the workDir.
+        return Toil.getToilWorkDir(self.config.workDir)
+
+    def format_std_out_err_path(self, toil_job_id: int, cluster_job_id: str, std: str) -> str:
         """
         Format path for batch system standard output/error and other files
         generated by the batch system itself.
 
-        Files will be written to the Toil work directory (which may
-        be on a shared file system) with names containing both the Toil and
-        batch system job IDs, for ease of debugging job failures.
+        Files will be written to the batch logs directory (--batchLogsDir,
+        defaulting to the Toil work directory) with names containing both the
+        Toil and batch system job IDs, for ease of debugging job failures.
 
         :param: int toil_job_id : The unique id that Toil gives a job.
         :param: cluster_job_id : What the cluster, for example, GridEngine, uses as its internal job id.
@@ -402,17 +427,17 @@ class BatchSystemSupport(AbstractBatchSystem):
         if self.config.noStdOutErr:
             return os.devnull
 
-        fileName: str = f'toil_{self.config.workflowID}.{toil_job_id}.{cluster_job_id}.{std}.log'
-        workDir: str = Toil.getToilWorkDir(self.config.workDir)
-        return os.path.join(workDir, fileName)
+        file_name: str = f'toil_{self.config.workflowID}.{toil_job_id}.{cluster_job_id}.{std}.log'
+        logs_dir: str = self.get_batch_logs_dir()
+        return os.path.join(logs_dir, file_name)
     
     def format_std_out_err_glob(self, toil_job_id: int) -> str:
         """
-        Get a glob string that will match all file paths generated by formatStdOutErrPath for a job.
+        Get a glob string that will match all file paths generated by format_std_out_err_path for a job.
         """
         file_glob: str = f'toil_{self.config.workflowID}.{toil_job_id}.*.log'
-        work_dir: str = Toil.getToilWorkDir(self.config.workDir)
-        return os.path.join(work_dir, file_glob)
+        logs_dir: str = self.get_batch_logs_dir()
+        return os.path.join(logs_dir, file_glob)
         
     @staticmethod
     def workerCleanup(info: WorkerCleanupInfo) -> None:
@@ -422,6 +447,7 @@ class BatchSystemSupport(AbstractBatchSystem):
         :param WorkerCleanupInfo info: A named tuple consisting of all the relevant information
                for cleaning up the worker.
         """
+        logger.debug('Attempting worker cleanup')
         assert isinstance(info, WorkerCleanupInfo)
         assert info.workflow_id is not None
         workflowDir = Toil.getLocalWorkflowDir(info.workflow_id, info.work_dir)
@@ -431,9 +457,11 @@ class BatchSystemSupport(AbstractBatchSystem):
         AbstractFileStore.shutdownFileStore(info.workflow_id, info.work_dir, info.coordination_dir)
         if info.clean_work_dir in ('always', 'onSuccess', 'onError'):
             if workflowDirContents in ([], [cacheDirName(info.workflow_id)]):
+                logger.debug('Deleting workflow directory %s', workflowDir)
                 shutil.rmtree(workflowDir, ignore_errors=True)
             if coordination_dir != workflowDir:
                 # No more coordination to do here either.
+                logger.debug('Deleting coordination directory %s', coordination_dir)
                 shutil.rmtree(coordination_dir, ignore_errors=True)
 
 class NodeInfo:
@@ -534,11 +562,7 @@ class InsufficientSystemResources(Exception):
         :param details: Any extra details about the problem that can be attached to the error.
         """
 
-        self.job_name : Optional[str] = None
-        if hasattr(requirer, 'jobName') and isinstance(getattr(requirer, 'jobName'), str):
-            # Keep the job name if any
-            self.job_name = cast(str, getattr(requirer, 'jobName'))
-
+        self.job_name : Optional[str] = str(requirer)
         self.resource = resource
         self.requested = cast(ParsedRequirement, getattr(requirer, resource))
         self.available = available
@@ -579,3 +603,171 @@ class InsufficientSystemResources(Exception):
 
         return ''.join(msg)
 
+
+class AcquisitionTimeoutException(Exception):
+    """To be raised when a resource request times out."""
+    def __init__(self, resource: str, requested: Union[int, float, Set[int]], available: Union[int, float, Set[int]]) -> None:
+        """
+        Creates an instance of this exception that indicates which resource is insufficient for
+        current demands, as well as the resources requested and actually available.
+        :param str resource: string representing the resource type
+        :param requested: the resources requested that resulted in this exception
+        :param available: the resources actually available
+        """
+        self.requested = requested
+        self.available = available
+        self.resource = resource
+
+
+class ResourcePool:
+    """
+    Represents an integral amount of a resource (such as memory bytes).
+    Amounts can be acquired immediately or with a timeout, and released.
+    Provides a context manager to do something with an amount of resource
+    acquired.
+    """
+    def __init__(self, initial_value: int, resource_type: str, timeout: float = 5) -> None:
+        super().__init__()
+        # We use this condition to signal everyone whenever some resource is released.
+        # We use its associated lock to guard value.
+        self.condition = Condition()
+        # This records how much resource is available right now.
+        self.value = initial_value
+        self.resource_type = resource_type
+        self.timeout = timeout
+
+    def acquireNow(self, amount: int) -> bool:
+        """
+        Reserve the given amount of the given resource.
+        Returns True if successful and False if this is not possible immediately.
+        """
+
+        with self.condition:
+            if amount > self.value:
+                return False
+            self.value -= amount
+            self.__validate()
+            return True
+
+    def acquire(self, amount: int) -> None:
+        """
+        Reserve the given amount of the given resource.
+        Raises AcquisitionTimeoutException if this is not possible in under
+        self.timeout time.
+        """
+        with self.condition:
+            startTime = time.time()
+            while amount > self.value:
+                if time.time() - startTime >= self.timeout:
+                    # This means the thread timed out waiting for the resource.
+                    raise AcquisitionTimeoutException(resource=self.resource_type,
+                                                      requested=amount, available=self.value)
+                # Allow self.timeout seconds to get the resource, else quit
+                # through the above if condition. This wait + timeout is the
+                # last thing in the loop such that a request that takes longer
+                # than self.timeout due to multiple wakes under the threshold
+                # are still honored.
+                self.condition.wait(timeout=self.timeout)
+            self.value -= amount
+            self.__validate()
+
+    def release(self, amount: int) -> None:
+        with self.condition:
+            self.value += amount
+            self.__validate()
+            self.condition.notify_all()
+
+    def __validate(self) -> None:
+        assert 0 <= self.value
+
+    def __str__(self) -> str:
+        return str(self.value)
+
+    def __repr__(self) -> str:
+        return "ResourcePool(%i)" % self.value
+
+    @contextmanager
+    def acquisitionOf(self, amount: int) -> Iterator[None]:
+        self.acquire(amount)
+        try:
+            yield
+        finally:
+            self.release(amount)
+
+
+class ResourceSet:
+    """
+    Represents a collection of distinct resources (such as accelerators).
+    Subsets can be acquired immediately or with a timeout, and released.
+    Provides a context manager to do something with a set of of resources
+    acquired.
+    """
+    def __init__(self, initial_value: Set[int], resource_type: str, timeout: float = 5) -> None:
+        super().__init__()
+        # We use this condition to signal everyone whenever some resource is released.
+        # We use its associated lock to guard value.
+        self.condition = Condition()
+        # This records what resources are available right now.
+        self.value = initial_value
+        self.resource_type = resource_type
+        self.timeout = timeout
+
+    def acquireNow(self, subset: Set[int]) -> bool:
+        """
+        Reserve the given amount of the given resource.
+        Returns True if successful and False if this is not possible immediately.
+        """
+
+        with self.condition:
+            if subset > self.value:
+                return False
+            self.value -= subset
+            return True
+
+    def acquire(self, subset: Set[int]) -> None:
+        """
+        Reserve the given amount of the given resource.
+        Raises AcquisitionTimeoutException if this is not possible in under
+        self.timeout time.
+        """
+        with self.condition:
+            startTime = time.time()
+            while subset > self.value:
+                if time.time() - startTime >= self.timeout:
+                    # This means the thread timed out waiting for the resource.
+                    raise AcquisitionTimeoutException(resource=self.resource_type,
+                                                      requested=subset, available=self.value)
+                # Allow self.timeout seconds to get the resource, else quit
+                # through the above if condition. This wait + timeout is the
+                # last thing in the loop such that a request that takes longer
+                # than self.timeout due to multiple wakes under the threshold
+                # are still honored.
+                self.condition.wait(timeout=self.timeout)
+            self.value -= subset
+
+    def release(self, subset: Set[int]) -> None:
+        with self.condition:
+            self.value |= subset
+            self.condition.notify_all()
+
+    def get_free_snapshot(self) -> Set[int]:
+        """
+        Get a snapshot of what items are free right now.
+        May be stale as soon as you get it, but you will need some kind of hint
+        to try and do an acquire.
+        """
+        return set(self.value)
+
+    def __str__(self) -> str:
+        return str(self.value)
+
+    def __repr__(self) -> str:
+        return "ResourceSet(%s)" % self.value
+
+    @contextmanager
+    def acquisitionOf(self, subset: Set[int]) -> Iterator[None]:
+        self.acquire(subset)
+        try:
+            yield
+        finally:
+            self.release(subset)
