@@ -20,20 +20,24 @@ import subprocess
 import time
 import traceback
 from argparse import ArgumentParser, _ArgumentGroup
-from contextlib import contextmanager
 from queue import Empty, Queue
-from threading import Condition, Event, Lock, Thread
+from threading import Event, Lock, Thread
 from typing import Dict, List, Optional, Set, Sequence, Tuple, Union
 
 import toil
 from toil import worker as toil_worker
-from toil.batchSystems.abstractBatchSystem import EXIT_STATUS_UNAVAILABLE_VALUE, BatchSystemSupport, UpdatedBatchJobInfo, InsufficientSystemResources
+from toil.batchSystems.abstractBatchSystem import (EXIT_STATUS_UNAVAILABLE_VALUE,
+                                                   BatchSystemSupport,
+                                                   ResourcePool,
+                                                   ResourceSet,
+                                                   UpdatedBatchJobInfo,
+                                                   InsufficientSystemResources)
 
 from toil.bus import ExternalBatchIdMessage
 from toil.batchSystems.options import OptionSetter
 
 from toil.common import SYS_MAX_SIZE, Config, Toil, fC
-from toil.job import JobDescription, AcceleratorRequirement, Requirer
+from toil.job import JobDescription, AcceleratorRequirement, accelerator_satisfies, Requirer
 from toil.lib.accelerators import get_individual_local_accelerators, get_restrictive_environment_for_local_accelerators
 from toil.lib.threading import cpu_count
 
@@ -77,16 +81,25 @@ class SingleMachineBatchSystem(BatchSystemSupport):
     physicalMemory = toil.physicalMemory()
 
     def __init__(
-        self, config: Config, maxCores: int, maxMemory: int, maxDisk: int
+        self, config: Config, maxCores: float, maxMemory: int, maxDisk: int, max_jobs: Optional[int] = None
     ) -> None:
         self.config = config
+
+        if max_jobs is None:
+            # Use the main limit for jobs from the config
+            self.max_jobs = self.config.max_jobs
+        else:
+            # Use the override (probably because we are the local batch system
+            # and not the main one).
+            self.max_jobs = max_jobs
+
         # Limit to the smaller of the user-imposed limit and what we actually
         # have on this machine for each resource.
         #
         # If we don't have up to the limit of the resource (and the resource
         # isn't the inlimited sentinel), warn.
         if maxCores > self.numCores:
-            if maxCores != SYS_MAX_SIZE:
+            if maxCores != SYS_MAX_SIZE and maxCores != float('inf'):
                 # We have an actually specified limit and not the default
                 logger.warning('Not enough cores! User limited to %i but we only have %i.', maxCores, self.numCores)
             maxCores = self.numCores
@@ -121,8 +134,9 @@ class SingleMachineBatchSystem(BatchSystemSupport):
 
         self.debugWorker = config.debugWorker
 
-        # A counter to generate job IDs and a lock to guard it
-        self.jobIndex = 0
+        # A counter to generate job IDs and a lock to guard it.
+        # We make sure to start this at 1 so that job IDs are never falsy.
+        self.jobIndex = 1
         self.jobIndexLock = Lock()
 
         # A dictionary mapping batch system IDs of submitted jobs to the command line
@@ -150,6 +164,8 @@ class SingleMachineBatchSystem(BatchSystemSupport):
 
         # Put them all organized by resource type
         self.resource_sources = [
+            # A pool representing available job slots
+            ResourcePool(self.max_jobs, 'job slots'),
             # A pool representing available CPU in units of minCores
             ResourcePool(int(self.maxCores / self.minCores), 'cores'),
             # A pool representing available memory in bytes
@@ -525,6 +541,8 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         algorithm, so if the job requires different kinds of accelerators, and
         some accelerators available can match multiple requirements, then it is
         possible that a solution will not be found.
+
+        Ignores accelerator model constraints.
         """
         accelerators_needed: Set[int] = set()
         accelerators_still_available = set(available_accelerator_ids)
@@ -536,8 +554,14 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                     # Check all the ones we haven't grabbed yet
                     # TODO: We'll re-check early ones against this requirement if it has a count of more than one.
                     candidate = self.accelerator_identities[candidate_index]
-                    if AcceleratorRequirement.satisfies(candidate, requirement):
-                        # If this accelerator can satisfy one unit of this requirement
+                    if accelerator_satisfies(candidate, requirement, ignore=['model']):
+                        # If this accelerator can satisfy one unit of this requirement.
+                        # We ignore model constraints because as a single
+                        # machine we can't really determine the models of
+                        # installed accelerators in a way consistent with how
+                        # workflows are going to name them, and usually people
+                        # use one model anyway.
+
                         # Say we want it
                         accelerators_needed.add(candidate_index)
                         accelerators_still_available.remove(candidate_index)
@@ -568,8 +592,8 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         startTime = time.time()
 
         # And what do we want from each resource in self.resource_sources?
-        # We know they go cores, memory, disk, accelerators.
-        resource_requests: List[Union[int, Set[int]]] = [coreFractions, jobMemory, jobDisk]
+        # We know they go job slot, cores, memory, disk, accelerators.
+        resource_requests: List[Union[int, Set[int]]] = [1, coreFractions, jobMemory, jobDisk]
 
         # Keep a reference to the accelerators separately
         accelerators_needed = None
@@ -844,184 +868,3 @@ class Info:
         self.popen = popen
         self.resources = resources
         self.killIntended = killIntended
-
-class AcquisitionTimeoutException(Exception):
-    """To be raised when a resource request times out."""
-    def __init__(self, resource: str, requested: Union[int, float, Set[int]], available: Union[int, float, Set[int]]):
-        """
-        Creates an instance of this exception that indicates which resource is insufficient for
-        current demands, as well as the resources requested and actually available.
-
-        :param str resource: string representing the resource type
-
-        :param requested: the resources requested that resulted in this exception
-
-        :param available: the resources actually available
-        """
-        self.requested = requested
-        self.available = available
-        self.resource = resource
-
-class ResourcePool:
-    """
-    Represents an integral amount of a resource (such as memory bytes).
-
-    Amounts can be acquired immediately or with a timeout, and released.
-
-    Provides a context manager to do something with an amount of resource
-    acquired.
-    """
-    def __init__(self, initial_value: int, resource_type: str, timeout=5):
-        super().__init__()
-        # We use this condition to signal everyone whenever some resource is released.
-        # We use its associated lock to guard value.
-        self.condition = Condition()
-        # This records how much resource is available right now.
-        self.value = initial_value
-        self.resource_type = resource_type
-        self.timeout = timeout
-
-    def acquireNow(self, amount):
-        """
-        Reserve the given amount of the given resource.
-
-        Returns True if successful and False if this is not possible immediately.
-        """
-
-        with self.condition:
-            if amount > self.value:
-                return False
-            self.value -= amount
-            self.__validate()
-            return True
-
-
-    def acquire(self, amount):
-        """
-        Reserve the given amount of the given resource.
-
-        Raises AcquisitionTimeoutException if this is not possible in under
-        self.timeout time.
-        """
-        with self.condition:
-            startTime = time.time()
-            while amount > self.value:
-                if time.time() - startTime >= self.timeout:
-                    # This means the thread timed out waiting for the resource.
-                    raise AcquisitionTimeoutException(resource=self.resource_type,
-                                                      requested=amount, available=self.value)
-                # Allow self.timeout seconds to get the resource, else quit
-                # through the above if condition. This wait + timeout is the
-                # last thing in the loop such that a request that takes longer
-                # than self.timeout due to multiple wakes under the threshold
-                # are still honored.
-                self.condition.wait(timeout=self.timeout)
-            self.value -= amount
-            self.__validate()
-
-    def release(self, amount):
-        with self.condition:
-            self.value += amount
-            self.__validate()
-            self.condition.notify_all()
-
-    def __validate(self):
-        assert 0 <= self.value
-
-    def __str__(self):
-        return str(self.value)
-
-    def __repr__(self):
-        return "ResourcePool(%i)" % self.value
-
-    @contextmanager
-    def acquisitionOf(self, amount):
-        self.acquire(amount)
-        try:
-            yield
-        finally:
-            self.release(amount)
-
-class ResourceSet:
-    """
-    Represents a collection of distinct resources (such as accelerators).
-
-    Subsets can be acquired immediately or with a timeout, and released.
-
-    Provides a context manager to do something with a set of of resources
-    acquired.
-    """
-    def __init__(self, initial_value: Set[int], resource_type: str, timeout: float = 5):
-        super().__init__()
-        # We use this condition to signal everyone whenever some resource is released.
-        # We use its associated lock to guard value.
-        self.condition = Condition()
-        # This records what resources are available right now.
-        self.value = initial_value
-        self.resource_type = resource_type
-        self.timeout = timeout
-
-    def acquireNow(self, subset: Set[int]):
-        """
-        Reserve the given amount of the given resource.
-
-        Returns True if successful and False if this is not possible immediately.
-        """
-
-        with self.condition:
-            if subset > self.value:
-                return False
-            self.value -= subset
-            return True
-
-    def acquire(self, subset: Set[int]):
-        """
-        Reserve the given amount of the given resource.
-
-        Raises AcquisitionTimeoutException if this is not possible in under
-        self.timeout time.
-        """
-        with self.condition:
-            startTime = time.time()
-            while subset > self.value:
-                if time.time() - startTime >= self.timeout:
-                    # This means the thread timed out waiting for the resource.
-                    raise AcquisitionTimeoutException(resource=self.resource_type,
-                                                      requested=subset, available=self.value)
-                # Allow self.timeout seconds to get the resource, else quit
-                # through the above if condition. This wait + timeout is the
-                # last thing in the loop such that a request that takes longer
-                # than self.timeout due to multiple wakes under the threshold
-                # are still honored.
-                self.condition.wait(timeout=self.timeout)
-            self.value -= subset
-
-    def release(self, subset: Set[int]):
-        with self.condition:
-            self.value |= subset
-            self.condition.notify_all()
-
-    def get_free_snapshot(self) -> Set[int]:
-        """
-        Get a snapshot of what items are free right now.
-        May be stale as soon as you get it, but you will need some kind of hint
-        to try and do an acquire.
-        """
-        return set(self.value)
-
-    def __str__(self):
-        return str(self.value)
-
-    def __repr__(self):
-        return "ResourceSet(%s)" % self.value
-
-    @contextmanager
-    def acquisitionOf(self, subset: Set[int]):
-        self.acquire(subset)
-        try:
-            yield
-        finally:
-            self.release(subset)
-
-
-
