@@ -22,6 +22,7 @@ from toil.batchSystems.abstractGridEngineBatchSystem import \
     AbstractGridEngineBatchSystem
 from toil.batchSystems.options import OptionSetter
 from toil.lib.misc import CalledProcessErrorStderr, call_command
+from toil.job import Requirer
 
 logger = logging.getLogger(__name__)
 
@@ -40,7 +41,7 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
             # -h for no header
             # --format to get jobid i, state %t and time days-hours:minutes:seconds
 
-            lines = call_command(['squeue', '-h', '--format', '%i %t %M']).split('\n')
+            lines = call_command(['squeue', '-h', '--format', '%i %t %M'], quiet=True).split('\n')
             for line in lines:
                 values = line.split()
                 if len(values) < 3:
@@ -61,12 +62,31 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                               jobID: int,
                               command: str,
                               jobName: str,
-                              job_environment: Optional[Dict[str, str]] = None) -> List[str]:
-            return self.prepareSbatch(cpu, memory, jobID, jobName, job_environment) + [f'--wrap={command}']
+                              job_environment: Optional[Dict[str, str]] = None,
+                              gpus: Optional[int] = None) -> List[str]:
+            return self.prepareSbatch(cpu, memory, jobID, jobName, job_environment, gpus) + [f'--wrap={command}']
 
         def submitJob(self, subLine):
             try:
-                output = call_command(subLine)
+                # Slurm is not quite clever enough to follow the XDG spec on
+                # its own. If the submission command sees e.g. XDG_RUNTIME_DIR
+                # in our environment, it will send it along (especially with
+                # --export=ALL), even though it makes a promise to the job that
+                # Slurm isn't going to keep. It also has a tendency to create
+                # /run/user/<uid> *at the start* of a job, but *not* keep it
+                # around for the duration of the job.
+                #
+                # So we hide the whole XDG universe from Slurm before we make
+                # the submission.
+                # Might as well hide DBUS also.
+                # This doesn't get us a trustworthy XDG session in Slurm, but
+                # it does let us see the one Slurm tries to give us.
+                no_session_environment = os.environ.copy()
+                session_names = [n for n in no_session_environment.keys() if n.startswith('XDG_') or n.startswith('DBUS_')]
+                for name in session_names:
+                    del no_session_environment[name]
+
+                output = call_command(subLine, env=no_session_environment)
                 # sbatch prints a line like 'Submitted batch job 2954103'
                 result = int(output.strip().split()[-1])
                 logger.debug("sbatch submitted job %d", result)
@@ -145,7 +165,7 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                     '--format', 'JobIDRaw,State,ExitCode',  # specify output columns
                     '-P',  # separate columns with pipes
                     '-S', '1970-01-01']  # override start time limit
-            stdout = call_command(args)
+            stdout = call_command(args, quiet=True)
 
             # Collect the job statuses in a dict; key is the job-id, value is a tuple containing
             # job state and exit status. Initialize dict before processing output of `sacct`.
@@ -154,7 +174,6 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                 job_statuses[job_id] = (None, None)
 
             for line in stdout.splitlines():
-                #logger.debug("%s output %s", args[0], line)
                 values = line.strip().split('|')
                 if len(values) < 3:
                     continue
@@ -190,7 +209,7 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
             if len(job_id_list) == 1:
                 args.append(str(job_id_list[0]))
 
-            stdout = call_command(args)
+            stdout = call_command(args, quiet=True)
 
             # Job records are separated by a blank line.
             if isinstance(stdout, str):
@@ -213,7 +232,6 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                 job = {}
                 for line in record.splitlines():
                     for item in line.split():
-                        #logger.debug("%s output %s", args[0], item)
                         # Output is in the form of many key=value pairs, multiple pairs on each line
                         # and multiple lines in the output. Each pair is pulled out of each line and
                         # added to a dictionary.
@@ -262,11 +280,13 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                           mem: int,
                           jobID: int,
                           jobName: str,
-                          job_environment: Optional[Dict[str, str]]) -> List[str]:
+                          job_environment: Optional[Dict[str, str]],
+                          gpus: Optional[int]) -> List[str]:
 
             #  Returns the sbatch command line before the script to run
             sbatch_line = ['sbatch', '-J', f'toil_job_{jobID}_{jobName}']
-
+            if gpus:
+                sbatch_line = sbatch_line[:1] + [f'--gres=gpu:{gpus}'] + sbatch_line[1:]
             environment = {}
             environment.update(self.boss.environment)
             if job_environment:
@@ -311,8 +331,8 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
             if cpu is not None:
                 sbatch_line.append(f'--cpus-per-task={math.ceil(cpu)}')
 
-            stdoutfile: str = self.boss.formatStdOutErrPath(jobID, '%j', 'out')
-            stderrfile: str = self.boss.formatStdOutErrPath(jobID, '%j', 'err')
+            stdoutfile: str = self.boss.format_std_out_err_path(jobID, '%j', 'out')
+            stderrfile: str = self.boss.format_std_out_err_path(jobID, '%j', 'err')
             sbatch_line.extend(['-o', stdoutfile, '-e', stderrfile])
 
             return sbatch_line
@@ -334,25 +354,25 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
                 pass  # slurm may return INVALID instead of a time
             return total_seconds
 
+    def _check_accelerator_request(self, requirer: Requirer) -> None:
+        for accelerator in requirer.accelerators:
+            if accelerator['kind'] != 'gpu':
+                raise InsufficientSystemResources(requirer, 'accelerators', details=
+                                                  [
+                                                      f'The accelerator {accelerator} could not be provided'
+                                                      'The Toil Slurm batch system only supports gpu accelerators at the moment.'
+                                                  ])
+
     ###
     ### The interface for SLURM
     ###
 
-    @classmethod
-    def getWaitDuration(cls):
-        # Extract the slurm batchsystem config for the appropriate value
-        lines = call_command(['scontrol', 'show', 'config']).split('\n')
-        time_value_list = []
-        for line in lines:
-            values = line.split()
-            if len(values) > 0 and (values[0] == "SchedulerTimeSlice" or values[0] == "AcctGatherNodeFreq"):
-                time_name = values[values.index('=')+1:][1]
-                time_value = int(values[values.index('=')+1:][0])
-                if time_name == 'min':
-                    time_value *= 60
-                # Add a 20% ceiling on the wait duration relative to the scheduler update duration
-                time_value_list.append(math.ceil(time_value*1.2))
-        return max(time_value_list)
+    # `scontrol show config` can get us the slurm config, and there are values
+    # SchedulerTimeSlice and AcctGatherNodeFreq in there, but
+    # SchedulerTimeSlice is for time-sharing preemtion and AcctGatherNodeFreq
+    # is for reporting resource statistics (and can be 0). Slurm does not
+    # actually seem to have a scheduling granularity or tick rate. So we don't
+    # implement getWaitDuration().
 
     @classmethod
     def add_options(cls, parser: Union[ArgumentParser, _ArgumentGroup]):
