@@ -233,7 +233,7 @@ class FileJobStore(AbstractJobStore):
             # This is the good case; the delete arrived in time.
             # If it didn't, we might go on to re-execute the already-finished job.
             # Anyway, this job doesn't really exist after all.
-            raise NoSuchJobException()
+            raise NoSuchJobException(job_id)
 
         # Pass along the current config, which is the JobStore's responsibility.
         job.assignConfig(self.config)
@@ -307,13 +307,19 @@ class FileJobStore(AbstractJobStore):
     def _copy_or_link(self, src_path, dst_path, symlink=False):
         # linking is not done be default because of issue #1755
         srcPath = self._extract_path_from_url(src_path)
-        if self.linkImports or symlink:
+        if self.linkImports and symlink:
             os.symlink(os.path.realpath(srcPath), dst_path)
         else:
             atomic_copy(srcPath, dst_path)
 
-    def _import_file(self, otherCls, uri, shared_file_name=None, hardlink=False, symlink=False):
+    def _import_file(self, otherCls, uri, shared_file_name=None, hardlink=False, symlink=True):
+        # symlink argument says whether the caller can take symlinks or not
+        # ex: if false, it implies the workflow cannot work with symlinks and thus will hardlink imports
+        # default is true since symlinking everything is ideal
         if issubclass(otherCls, FileJobStore):
+            if os.path.isdir(uri.path):
+                # Don't allow directories (unless someone is racing us)
+                raise IsADirectoryError(f"URI {uri} points to a directory but a file was expected")
             if shared_file_name is None:
                 executable = os.stat(uri.path).st_mode & stat.S_IXUSR != 0
                 absPath = self._get_unique_file_path(uri.path)  # use this to get a valid path to write to in job store
