@@ -1,4 +1,5 @@
 import os
+import tempfile
 
 from toil.common import Toil
 from toil.job import Job, PromisedRequirement
@@ -25,7 +26,9 @@ def analysisJob(job, fileStoreID, cores=2):
 
 
 if __name__ == "__main__":
-    options = Job.Runner.getDefaultOptions("./toilWorkflowRun")
+    jobstore: str = tempfile.mkdtemp("tutorial_requirements")
+    os.rmdir(jobstore)
+    options = Job.Runner.getDefaultOptions(jobstore)
     options.logLevel = "INFO"
     options.clean = "always"
 
